﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;
using FlowBasis.Json;
using System.Collections;

namespace FlowBasisUnitTests.Json
{
    
    public class JObjectTests
    {
        [Test]
        public void Should_Return_Null_For_Missing_Value()
        {
            dynamic jObject = new JObject();
            jObject.someValue = "hello";

            Assert.IsNull(jObject.missingValue);
            Assert.IsNull(((JObject)jObject)["missingValue"]);
        }


        [Test]
        [Row("hello")]
        [Row("\"double quoted\"")]
        [Row("'single quoted'")]
        [Row(0)]
        [Row(-3)]
        [Row(5)]
        [Row(true)]
        [Row(false)]        
        public void Should_Roundtrip_Strings_Ints_And_Booleans(object value)
        {
            string json = JObject.Stringify(value);
            object deserializedValue = JObject.Parse(json);

            Assert.AreEqual(value, deserializedValue);
        }


        [Test]
        [Row(5.5f)]
        [Row(5.25d)] 
        public void Should_Roundtrip_Floats_And_Doubles_To_Decimal(object value)
        {
            string json = JObject.Stringify(value);
            object deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<decimal>(deserializedValue);

            Assert.AreEqual(Convert.ToDecimal(value), (decimal)deserializedValue);
        }


        [Test]
        public void Should_Roundtrip_DateTimeUtc()
        {
            DateTime date = new DateTime(2010, 5, 4, 10, 3, 23).AddMilliseconds(256).ToUniversalTime();
            string json = JObject.Stringify(date);

            DateTime resultDate = (DateTime)JObject.Parse(json);
            Assert.AreEqual(date, resultDate);
            Assert.AreEqual(DateTimeKind.Utc, resultDate.Kind);
        }


        [Test]   
        public void Should_Roundtrip_Arrays_To_Array_List()
        {
            // Array of mixed values.
            string json = JObject.Stringify(new object[] { 2, 3, "hello" });
            object deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<ArrayList>(deserializedValue);

            var list = (ArrayList)deserializedValue;
            Assert.AreEqual(3, list.Count);
            Assert.AreEqual(2, list[0]);
            Assert.AreEqual(3, list[1]);
            Assert.AreEqual("hello", list[2]);

            // Array of ints.
            json = JObject.Stringify(new object[] { -3, 12 });
            deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<ArrayList>(deserializedValue);

            list = (ArrayList)deserializedValue;
            Assert.AreEqual(2, list.Count);
            Assert.AreEqual(-3, list[0]);
            Assert.AreEqual(12, list[1]);

            // Empty array.
            json = JObject.Stringify(new object[] {});
            deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<ArrayList>(deserializedValue);

            list = (ArrayList)deserializedValue;
            Assert.AreEqual(0, list.Count);
        }


        [Test]
        public void Should_Roundtrip_Arrays_Of_Arrays_To_Array_Lists()
        {
            // Array of mixed values.
            string json = JObject.Stringify(new object[] { -122, new int[] { 6, -1}, new object[] { "hello", "world" } });
            object deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<ArrayList>(deserializedValue);

            var list = (ArrayList)deserializedValue;
            Assert.AreEqual(3, list.Count);
            Assert.AreEqual(-122, list[0]);

            Assert.IsInstanceOfType<ArrayList>(list[1]);
            var listChild1 = (ArrayList)list[1];
            Assert.AreEqual(2, listChild1.Count);
            Assert.AreEqual(6, listChild1[0]);
            Assert.AreEqual(-1, listChild1[1]);

            Assert.IsInstanceOfType<ArrayList>(list[2]);
            var listChild2 = (ArrayList)list[2];
            Assert.AreEqual(2, listChild2.Count);
            Assert.AreEqual("hello", listChild2[0]);
            Assert.AreEqual("world", listChild2[1]);
        }

        [Test]
        public void Should_Roundtrip_Array_Of_JObject()
        {
            // Array of mixed values.
            dynamic jObject = new JObject();
            jObject.someNum = 42;
            jObject.someStr = "hello";

            string json = JObject.Stringify(new object[] { jObject });
            object deserializedValue = JObject.Parse(json);

            Assert.IsInstanceOfType<ArrayList>(deserializedValue);

            var list = (ArrayList)deserializedValue;
            Assert.AreEqual(1, list.Count);

            Assert.IsInstanceOfType<JObject>(list[0]);
            dynamic deserializedJObject = list[0];
            Assert.AreEqual(42, deserializedJObject.someNum);
            Assert.AreEqual("hello", deserializedJObject.someStr);
        }


        [Test]
        public void Should_Roundtrip_Contained_Array_To_Array_List()
        {
            dynamic jObject = new JObject();
            jObject.intArray = new int[] { 2, 3, 5 };            

            string json = JObject.Stringify(jObject);
            object deserializedValue = JObject.Parse(json);
            Assert.IsInstanceOfType<JObject>(deserializedValue);

            dynamic deserializedJObject = deserializedValue;

            Assert.IsInstanceOfType<ArrayList>(deserializedJObject.intArray);

            ArrayList list = (ArrayList)deserializedJObject.intArray;
            Assert.AreEqual(3, list.Count);
            Assert.AreEqual(2, list[0]);
            Assert.AreEqual(3, list[1]);
            Assert.AreEqual(5, list[2]);
        }


        [Test]
        public void Should_Roundtrip_Nested_JObject()
        {
            dynamic jObject = new JObject();
            jObject.level1 = new JObject();
            jObject.level1.someNum = 12;
            jObject.level1.level2 = new JObject();
            jObject.level1.level2.someStr = "hello";

            string json = JObject.Stringify(jObject);
            object deserializedValue = JObject.Parse(json);
            Assert.IsInstanceOfType<JObject>(deserializedValue);

            dynamic deserializedJObject = deserializedValue;

            Assert.IsInstanceOfType<JObject>(deserializedJObject.level1);
            Assert.IsInstanceOfType<JObject>(deserializedJObject.level1.level2);

            Assert.AreEqual(12, deserializedJObject.level1.someNum);
            Assert.AreEqual("hello", deserializedJObject.level1.level2.someStr);
        }


        
    }

}
